@@ -23,8 +23,6 @@
  ***************************************************************************/
 #include "first.h"
 
-#include "memdebug.h"
-
 struct chunk_data {
   int remains;
   int print_content;
@@ -98,27 +96,27 @@ static long chunk_end(void *ptr)
 
 static CURLcode test_lib576(const char *URL)
 {
-  CURL *handle = NULL;
+  CURL *curl = NULL;
   CURLcode res = CURLE_OK;
   struct chunk_data chunk_data = {0, 0};
   curl_global_init(CURL_GLOBAL_ALL);
-  handle = curl_easy_init();
-  if(!handle) {
+  curl = curl_easy_init();
+  if(!curl) {
     res = CURLE_OUT_OF_MEMORY;
     goto test_cleanup;
   }
 
-  test_setopt(handle, CURLOPT_URL, URL);
-  test_setopt(handle, CURLOPT_WILDCARDMATCH, 1L);
-  test_setopt(handle, CURLOPT_CHUNK_BGN_FUNCTION, chunk_bgn);
-  test_setopt(handle, CURLOPT_CHUNK_END_FUNCTION, chunk_end);
-  test_setopt(handle, CURLOPT_CHUNK_DATA, &chunk_data);
+  test_setopt(curl, CURLOPT_URL, URL);
+  test_setopt(curl, CURLOPT_WILDCARDMATCH, 1L);
+  test_setopt(curl, CURLOPT_CHUNK_BGN_FUNCTION, chunk_bgn);
+  test_setopt(curl, CURLOPT_CHUNK_END_FUNCTION, chunk_end);
+  test_setopt(curl, CURLOPT_CHUNK_DATA, &chunk_data);
 
-  res = curl_easy_perform(handle);
+  res = curl_easy_perform(curl);
 
 test_cleanup:
-  if(handle)
-    curl_easy_cleanup(handle);
+  if(curl)
+    curl_easy_cleanup(curl);
   curl_global_cleanup();
   return res;
 }
